#ifndef _MIL1553_DRIVER_API_H
#define _MIL1553_DRIVER_API_H

#define MIL1553UDDEV                      0xA459

// Device versions
#define MIL1553_SIMPLE_DEVICE_VERSION     0
#define MIL1553_DEVTYPE_NONE              0

/////////////////////////////////////////////////////////////////////////////
// Interrupts Status
#define INT_HDAT	1
#define INT_QDAT	2
#define RT_INT_MC	3
#define RT_INT_ERR	4
#define RT_INT_REN	5
#define RT_INT_TEN	6
#define INT_BC		7

#define MIL1553_INT_DATA_HALFBUF        0x00000001UL  // data buffer 1/2 filling
#define MIL1553_INT_DATA_QUARTBUF       0x00000002UL  // data buffer 1/16 filling
#define MIL1553_INT_FLASH               0x00000004UL
#define MIL1553_INT_RT_TEN              0x00000010UL
#define MIL1553_INT_RT_REN		  		0x00000020UL
#define MIL1553_INT_BC			  		0x00000040UL

//RT mask
#define RT_INT_TEN_MASK		0x1
#define RT_INT_REN_MASK 	0x2
#define RT_INT_ERR_MASK 	0x4
#define RT_INT_MC_MASK 		0x8

#define MIL1553_INT_ALL                   0x00007BUL
#define MIL1553_INT_RT                    0x000030UL
#define MIL1553_INT_MASK_ALL              0x00007BUL
#define MIL1553_INT_MASK_RT               0x00000FUL

/////////////////////////////////////////////////////////////////////////////
// Registers

/////////////////////////////////////////////////////////////////////////////
// I/O extended codes

#define CTL_CODE(DeviceType, Function, Method, Access) (                 \
    ((DeviceType) << 16) | ((Access) << 14) | ((Function) << 2) | (Method) )

#define METHOD_BUFFERED                 0
#define METHOD_IN_DIRECT                1
#define METHOD_OUT_DIRECT               2
#define METHOD_NEITHER                  3

#define FILE_ANY_ACCESS                 0
#define FILE_READ_ACCESS                ( 0x0001 )    // file & pipe
#define FILE_WRITE_ACCESS               ( 0x0002 )    // file & pipe


#define IOCTL_MIL1553DEV_GET_DEVICE_VER \
  CTL_CODE(MIL1553UDDEV , 0x900, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
#define IOCTL_MIL1553DEV_GET_INT_FLAGS \
  CTL_CODE(MIL1553UDDEV , 0x908, METHOD_BUFFERED, FILE_READ_ACCESS)

#define IOCTL_MIL1553DEV_ENABLE_SUBADDR \
  CTL_CODE(MIL1553UDDEV , 0x912, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
#define IOCTL_MIL1553DEV_DISABLE_SUBADDR \
  CTL_CODE(MIL1553UDDEV , 0x913, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
#define IOCTL_MIL1553DEV_WRITE_SUBADDR \
  CTL_CODE(MIL1553UDDEV , 0x914, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
#define IOCTL_MIL1553DEV_READ_SUBADDR \
  CTL_CODE(MIL1553UDDEV , 0x915, METHOD_BUFFERED, FILE_READ_ACCESS)

#define IOCTL_MIL1553DEV_READ_SUBADDR_BUF \
  CTL_CODE(MIL1553UDDEV , 0x916, METHOD_BUFFERED, FILE_READ_ACCESS)
  
#define IOCTL_MIL1553DEV_WRITE_SUBADDR_BUF \
  CTL_CODE(MIL1553UDDEV , 0x917, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
#define IOCTL_MIL1553DEV_GET_RESOURCE_INFO \
  CTL_CODE(MIL1553UDDEV , 0x950, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_MIL1553DEV_CLEAR_DMA \
  CTL_CODE(MIL1553UDDEV , 0x953, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_MIL1553DEV_READ_BAR \
  CTL_CODE(MIL1553UDDEV , 0x954, METHOD_BUFFERED, FILE_READ_ACCESS)

#define IOCTL_MIL1553DEV_WRITE_BAR \
  CTL_CODE(MIL1553UDDEV , 0x955, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
#define IOCTL_MIL1553DEV_GET_NBLOCK_RAW_DMA \
  CTL_CODE(MIL1553UDDEV , 0x957, METHOD_BUFFERED, FILE_READ_ACCESS)
  
#define IOCTL_MIL1553DEV_DMA_COUNT \
  CTL_CODE(MIL1553UDDEV , 0x966, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_MIL1553DEV_READ_DMA_DATABUF \
  CTL_CODE(MIL1553UDDEV , 0x967, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_MIL1553DEV_READ_DMA_STATUS \
  CTL_CODE(MIL1553UDDEV , 0x968, METHOD_BUFFERED, FILE_ANY_ACCESS)  

#define IOCTL_MIL1553DEV_READ_DMA_DATA_BLOCKS \
  CTL_CODE(MIL1553UDDEV , 0x96B, METHOD_OUT_DIRECT, FILE_READ_ACCESS)
  
#define IOCTL_MIL1553DEV_READ_DATABUF \
  CTL_CODE(MIL1553UDDEV , 0x96C, METHOD_OUT_DIRECT, FILE_READ_ACCESS)

#define IOCTL_MIL1553DEV_WRITE_DATABUF \
  CTL_CODE(MIL1553UDDEV , 0x96D, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
#define IOCTL_MIL1553DEV_WRITE_REG  \
  CTL_CODE(MIL1553UDDEV , 0x97D, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_MIL1553DEV_READ_REG    \
  CTL_CODE(MIL1553UDDEV , 0x97E, METHOD_BUFFERED, FILE_ANY_ACCESS)
  
#define IOCTL_MIL1553DEV_REGISTER_INTERRUPT \
	CTL_CODE(MIL1553UDDEV, 0x97F, METHOD_BUFFERED, FILE_ANY_ACCESS)
	
#define IOCTL_MIL1553DEV_DISABLE_INTERRUPT  \
	CTL_CODE(MIL1553UDDEV , 0x980, METHOD_BUFFERED, FILE_WRITE_ACCESS)
  
/////////////////////////////////////////////////////////////////////////////
//Structs
//IOCTL_MIL1553DEV_GET_DEVICE_VER
typedef struct _MIL1553_GET_REV_OUT{
  UINT32 m_nRev;
} MIL1553_GET_REV_OUT;

//IOCTL_MIL1553DEV_GET_NBLOCK_RAW_DMA
typedef struct _MIL1553_GET_NBLOCK_DMA_OUT{
  UINT32 m_nValue;
} MIL1553_GET_NBLOCK_DMA_OUT;

//IOCTL_MIL1553DEV_DMA_COUNT
typedef struct _MIL1553_DMA_COUNT_OUT{
  UINT32 m_nCount;
} MIL1553_DMA_COUNT_OUT;

//IOCTL_MIL1553DEV_READ_DMA_STATUS 
typedef struct _MIL1553_READ_DMA_STATUS{
  UINT32 m_bStatus;
} MIL1553_READ_DMA_STATUS;

//IOCTL_MIL1553DEV_GET_INT_FLAGS
typedef struct _MIL1553_GET_INT_FLAGS_OUT{
  UINT32 m_nIntValue;
  UINT32 m_nRTValue;
} MIL1553_GET_INT_FLAGS_OUT;

//IOCTL_MIL1553DEV_DISABLE_SUBADDR
//IOCTL_MIL1553DEV_ENABLE_SUBADDR
typedef struct _MIL1553_SUBADDR_IN{
  UINT32 m_nRT;
  UINT32 m_nNum;
} MIL1553_SUBADDR_IN;

//IOCTL_MIL1553DEV_READ_SUBADDR
//IOCTL_MIL1553DEV_WRITE_SUBADDR
typedef struct _MIL1553_WRRD_SUBADDR_IN{
  UINT32 m_nRT;
  UINT32 m_nNum;
  UINT32 m_nBufNum;
  UINT32 m_nData;
} MIL1553_WRRD_SUBADDR_IN;

//IOCTL_MIL1553DEV_WRITE_SUBADDR_BUF
typedef struct _MIL1553_WR_SUBADDR_IN{
  UINT32 m_nCh;
  UINT32 m_nNum;
  UINT32 m_nBufNum;
  UINT32 m_nData[16];
} MIL1553_WR_SUBADDR_IN;

//IOCTL_MIL1553DEV_READ_SUBADDR_BUF
typedef struct _MIL1553_RD_SUBADDR_IN{
  UINT32 m_nCh;
  UINT32 m_nNum;
  UINT32 m_nBufNum;
} MIL1553_RD_SUBADDR_IN;

//IOCTL_MIL1553DEV_GET_STATUS
typedef struct _MIL1553_GET_STATUS_IN{
  UINT32 m_nTime;
} MIL1553_GET_STATUS_IN;

// IOCTL_MIL1553DEV_WRITE_REG
typedef struct _MIL1553_WRITE_REG_IN{
  UINT32 m_nAdress;
  UINT32 m_nData;
} MIL1553_WRITE_REG_IN;

typedef struct _MIL1553_WRITE_REG_OUT{
  UINT32 m_nData;
} MIL1553_WRITE_REG_OUT;

//IOCTL_MIL1553DEV_READ_REG
typedef struct _MIL1553_READ_REG_IN{
  UINT32 m_nAdress;
} MIL1553_READ_REG_IN;

typedef struct _MIL1553_READ_REG_OUT{
  UINT32 m_nData;
} MIL1553_READ_REG_OUT;

// IOCTL_MIL1553DEV_GET_RESOURCE_INFO
typedef struct _MIL1553_GET_RESOURCE_INFO_OUT{
  UINT32 m_nBus;
  UINT32 m_nSlot;
  UINT32 m_nFunction;
  UINT32 m_nDataBufPhysAddr;
  UINT32 m_nDataBufSize;
  UINT32 m_BarPhysAddr;
  UINT8  m_szCompileDate[128];
  struct {
    UINT32 m_nBootFlashPageSize;     // in bytes
    UINT32 m_nBootFlashSectorSize;   // in bytes
    UINT32 m_nBootFlashAreaSize;     // in bytes
    UINT32 m_nBootFlashTotalSize;    // in bytes
    UINT32 m_nBootFlashEraseTimeout; // in seconds
  } BootFlashInfo;
} MIL1553_GET_RESOURCE_INFO_OUT;

typedef struct _MIL1553_GET_DEVICE_TYPE_OUT{
  UINT32 m_nType;
} MIL1553_GET_DEVICE_TYPE_OUT;

// IOCTL_MIL1553DEV_READ_BAR
typedef struct _MIL1553_READ_BAR_IN{
  UINT32 m_nBar;
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
} MIL1553_READ_BAR_IN;

// IOCTL_MIL1553DEV_WRITE_BAR
typedef struct _MIL1553_WRITE_BAR_IN
{
  UINT32 m_nBar;
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
  UINT8 m_Data[0];       // array of bytes
} MIL1553_WRITE_BAR_IN;

// IOCTL_MIL1553DEV_READ_DMA_DATA_BLOCKS
typedef struct _MIL1553_READ_DMA_DATA_BLOCKS_IN{
  UINT32 m_nBlocks;        // Size (in bytes)
} MIL1553_READ_DMA_DATA_BLOCKS_IN;

// IOCTL_MIL1553_READ_DATABUF
typedef struct _MIL1553_READ_DATABUF_IN{
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
} MIL1553_READ_DATABUF_IN;

// IOCTL_MIL1553DEV_WRITE_DATABUF
typedef struct _MIL1553_WRITE_DATABUF_IN{
  UINT32 m_nOffset;      // Offset of data (in bytes)
  UINT32 m_nSize;        // Size (in bytes)
  UINT8 m_Data[0];       // array of bytes
} MIL1553_WRITE_DATABUF_IN;

// IOCTL_REGISTER_EVENT
typedef struct _MIL1553_REG_EVT_NOTIFICATION{
  HANDLE hEvent;
} MIL1553_REG_EVT_NOTIFICATION_IN;

//IOCTL_MIL1553DEV_REGISTER_EVENT
typedef struct _MIL1553_INT_NTFCN_IN{
	UINT32 m_nInt;
	HANDLE m_hEvt;
} MIL1553UD_INT_NTFCN_IN;

/////////////////////////////////////////////////////////////////////////////
// Macros
#define GetAbsoluteDecimation(ch, pch, out){                                   \
  if((ch == 0)                                                                 \
     || (pch[ch].m_nDdcDataSource == MIL1553_DDC_DATA_SOURCE_REALTIME)         \
     || (pch[0].m_nDdcCicDecimation == 0))                                     \
    out = pch[ch].m_nDdcCicDecimation;                                         \
  else                                                                         \
    out = pch[0].m_nDdcCicDecimation * pch[ch].m_nDdcCicDecimation;            \
  if(out == 0)                                                                 \
    out = 1;                                                                   \
}
#endif