
#ifndef _MIL1553_DEVICE_UD_H
#define _MIL1553_DEVICE_UD_H

#ifndef sprintf_s
#define sprintf_s sprintf
#endif

#include "StdInterfaces.h"
#include <string>
#include <vector>
#include "MIL1553Api.h"

/////////////////////////////////////////////////////////////////////////////
// Definitions

const char MIL1553UD_DRIVER_MASKNAME[] = "\\\\.\\Mil1553UD%d";

#define MIL1553UD_LIB_NUMOF_RETRIES       20

/////////////////////////////////////////////////////////////////////////////
// Class CMIL1553DeviceEx

class CMIL1553DeviceUD{
public:

  CMIL1553DeviceUD();
  virtual ~CMIL1553DeviceUD();

  //open/close
  BOOL Open(int nIndex);
  BOOL IsOpen() { return m_bOpen; }
  void Close();
  void Attach(HANDLE hDevice);
  HANDLE Detach();
  HANDLE GetHandle();

  // Operations
  BOOL GetDatabufSize(UINT32 *pBufSize);
  BOOL GetDatabufPhysAddr(UINT32 *pBufAddr);
  BOOL GetBarPhysAddr(UINT32 *pBarAddr);
  BOOL GetRevision(UINT32 *Revision);
  BOOL GetPCILocation(UINT32 *pBus, UINT32 *pSlot);
  BOOL GetDriverCompileDate(std::string *pStr);


  //Interrupt flags, buffered IO, delayed downlink
  UINT32 Register_Interrupt(UINT32 InterruptName, HANDLE hEvent);
  UINT32 DisableInterrupt(UINT32 InterruptName);
  BOOL GetIntFlags(UINT32 *nIntValue);
  BOOL GetIntFlagsRT(UINT32 *nIntValue);

  //Direct Bar access
  BOOL ReadBar(UINT32 nAddr, void *pData, UINT32 nSize);
  BOOL ReadReg(UINT32 nAddr, UINT32 *pData);
  BOOL WriteBar(UINT32 nAddr, const void *pData, UINT32 nSize);
  BOOL WriteReg(UINT32 nAddr, UINT32 pData);

  //Read DMA data
  UINT32 ReadDMADataOneBlock(void *pData);
  BOOL   ReadDMADataBlocks(void *pData, UINT32 nBlocks, UINT32 *pBlocks);
  BOOL   GetNBlockRawDMA(UINT32 *nValue);
  BOOL   GetDMAReadyStatus(UINT32 *bFlag);
  BOOL   GetProgCount(UINT32 *nCount);
  BOOL   ClearDMA();

  //Common Commands
  BOOL EnableDMA();
  BOOL DisableDMA();
  BOOL ReadFreeTimer(UINT32 *pData);
  BOOL ResetFreeTimer();
  BOOL ReadHW_STAT_REG(UINT32 *pData1, UINT32 *pData2);
  BOOL SetMode(UINT32 nMode);
  BOOL WorkEnable(bool bFlag);
  BOOL SetRTAddress(UINT32 Address);
  UINT32 BusSelection(UINT32 nMode);
  UINT32 EnableSubAddr(UINT32 nRT, UINT32 nNum);
  UINT32 DisableSubAddr(UINT32 nNum);
  UINT32 WriteSubAddrBuf(UINT32 nBufNum, UINT32 nNum, UINT32 *pData, UINT32 nSize);
  UINT32 ReadSubAddrBuf(UINT32 nBufNum, UINT32 nNum, UINT32 *pData, UINT32 nSize);
  UINT32 SetTrProgMode(UINT32 nNum);
  UINT32 SetTrAutoMode(UINT32 nNum);
  UINT32 SetTrBufBitReady(UINT32 nNum, UINT32 nBuf);
  UINT32 ReadTrBufStatus(UINT32 nNum, UINT32 nBuf, BOOL *bReadyFlag);

  //RT Commands
  BOOL SetRT_T1_TRCK(UINT32 nTimer);
  BOOL SetRT_TCK(UINT32 nTimer);
  BOOL SetRT_T1_RCVCK(UINT32 nTimer);
  BOOL SetMC16_CSR(BOOL bFlag);
  BOOL SetTX_OAC(BOOL bFlag);
  BOOL SetSET_INSTR(BOOL bFlag);
  BOOL SetSET_SR(BOOL bFlag);
  BOOL SetSET_BSY(BOOL bFlag);
  BOOL SetSET_SF(BOOL bFlag);
  BOOL SetSET_TF(BOOL bFlag);

 
  //BC Commands
  BOOL SetBC_TCK(UINT32 nTime);
  BOOL SetBC_T1_TRCK(UINT32 nTime);
  BOOL SetBC_T1_RCVCK(UINT32 nTime);
  BOOL SetTIME_UTIL_REG_PCI(UINT32 nTime);
  BOOL SetBCTTA(UINT32 nMode);
  BOOL SetTIME_SEL(BOOL bFlag);
  BOOL SetBCSTRT(BOOL bFlag);
  BOOL SetBCRE(BOOL bFlag);
  BOOL SetBC2RE(BOOL bFlag);
  BOOL SetBCR1A(BOOL bFlag);
  BOOL SetBCR2A(BOOL bFlag);
  BOOL SetBCRSB(BOOL bFlag);
  BOOL SetMENDV(BOOL bFlag);
  BOOL SetBSYNDV(BOOL bFlag);
  BOOL SetBCTTM(BOOL bFlag);
  BOOL SetTTSYNEN(BOOL bFlag);
  BOOL SetETTSYN(BOOL bFlag);
  BOOL SetBCRME(BOOL bFlag);

  
  //Flash
  UINT32 AutoLoadingAfterStartOn(UINT32 nArea);
  UINT32 AutoLoadingAfterStartOff();
  UINT32 WriteRegsValueToEEPROM(UINT32 nArea);
  UINT32 ReadRegsValueFromEEPROM(UINT32 nArea);
  


protected:
	BOOL ProcessDeviceIOControl(
    UINT32 dwIoControlCode,
    LPVOID lpInBuffer,
    UINT32 nInBufferSize,
    LPVOID lpOutBuffer,
    UINT32 nOutBufferSize,
    LPDWORD lpBytesReturned,
    UINT32 nTimeoutMs = INFINITE);

  BOOL m_bOpen;
  HANDLE m_hDevice;
  BOOL m_bOverlapped;
  OVERLAPPED m_ReadOverlapped;
  OVERLAPPED m_IOOverlapped;
  int m_nIndex;
  UINT32 m_nDeviceType;

  UINT8  m_simDeviceVersion;
  BOOL    m_bExchangeStarted;
  BOOL    m_bBufferedIO;
  BOOL    m_bDelayedDownlink;

};

#endif
